unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  ImgList, Buttons;

type
  TMainForm = class(TForm)
    ControlPanel: TPanel;
    DIOPanel: TPanel;
    CommandEdit: TEdit;
    LevelImages: TImageList;
    ClockButton2: TSpeedButton;
    ClockButton1: TSpeedButton;
    ClockButton0: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure CommandEditKeyPress(Sender: TObject; var Key: Char);
    procedure DIOImageClick(Sender: TObject);
    procedure ClockButtonClick(Sender: TObject);
  protected
    procedure SetByte(Index: Integer);
    //procedure GetByte(Index: Integer);
    procedure SetClock(Index: Integer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  ACCES32, Registry;

const
  HighDIO = 191;
  ActivityMask = $70;

var
  DIOImage: array[0..HighDIO] of TImage;
  DIOLabel: array[0..HighDIO] of TLabel;
  LevelBitmap: array[0..1] of TBitmap;
  ClockButton: array[0..2] of TSpeedButton;

var
  Base: Word;

type
  TPCI_COMMON_CONFIG = record
     VendorID : word;
     DeviceID : word;
     Command : word;
     Status : word;
     RevisionID : byte;
     ProgIf : byte;
     SubClass : byte;
     BaseClass : byte;
     CacheLineSize : byte;
     LatencyTimer : byte;
     HeaderType : byte;
     BIST : byte;
     BaseAddresses : Array[0..5] of longint;
     Reserved1 : Array[0..1] of longint;
     RomBaseAddress : longint;
     Reserved2 : Array[0..1] of longint;
     InterruptLine : byte;
     InterruptPin : byte;
     MinimumGrant : byte;
     MaximumLatency : byte;
  end;

function IntToBin(I, MinBits: LongWord): String;
var
  DoneBits: LongWord;

begin
  Result := '';
  DoneBits := 0;
  repeat
    Result := Char(Byte('0') + (I and 1)) + Result;
    I := I shr 1;
    Inc(DoneBits);
  until (I = 0) and (DoneBits >= MinBits);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  I, Num: Integer;
  PCIBuf: array of TPCI_COMMON_CONFIG;
  DriverRegistry: TRegistry;

begin
  DriverRegistry := TRegistry.Create;
  DriverRegistry.RootKey := HKEY_LOCAL_MACHINE;

  DriverRegistry.OpenKey('Software\PCIFIND\NTioPCI\Parameters', False);
  try
    Num := DriverRegistry.ReadInteger('NumDevices');
  except
    on ERegistryException do Num := 0;
  end;
  if (Num > 0) then begin
    SetLength(PCIBuf, Num);
    DriverRegistry.ReadBinaryData('PCICommonConfig', PCIBuf[0], Num * SizeOf(PCIBuf[0]));
  end;

  Base := 0;
  for I := 0 to Num - 1 do with PCIBuf[I] do begin
    case (DeviceID) of
      $0C50, $0C51, $0E51, $0E52: Base := BaseAddresses[2] AND $FFFE;
      $0E60: Base := BaseAddresses[2] AND $FFFE;
      else Continue;
    end;
    Break;
  end;
  DriverRegistry.Free;
  if Base = 0 then raise Exception.Create('PCI-DIO24D/H not found.');

  LevelBitmap[0] := TBitmap.Create;
  LevelImages.GetBitmap(0, LevelBitmap[0]);
  LevelBitmap[1] := TBitmap.Create;
  LevelImages.GetBitmap(1, LevelBitmap[1]);

  for I := 0 to HighDIO do begin
    DIOImage[I] := TImage.Create(Self);
    DIOLabel[I] := TLabel.Create(Self);
    with DIOImage[I] do begin
      Parent := DIOPanel;
      Left := (I div 8) * 30 + 5;
      Top := (I mod 8) * 31 + 2;
      Width := 20;
      Height := 20;
      Stretch := False;
      Tag := I;
      OnClick := DIOImageClick;
      OnDblClick := DIOImageClick;
      Picture.Assign(LevelBitmap[1]);
    end;
    with DIOLabel[I] do begin
      AutoSize := False;
      Parent := DIOPanel;
      Left := (I div 8) * 30;
      Top := (I mod 8) * 31 + 20;
      Width := 30;
      Height := 12;
      Alignment := taCenter;
      Layout := tlTop;
      Transparent := True;
      Caption := IntToStr(I);
      Tag := 1;
      Font.Name := 'Tahoma';
      Font.Size := 7;
    end;
  end;

  ClientWidth := 30 * ((HighDIO + 1) div 8);
  ClientHeight := 28 + 4 + 31 * 8;

  ClockButton[0] := ClockButton0;
  ClockButton[1] := ClockButton1;
  ClockButton[2] := ClockButton2;

  OutPortB(Base + 3, $82); //A out, B in, CLo out, CHi out
  OutPortB(Base + 0, $FF);
  OutPortB(Base + 2, ActivityMask);
  for I := 0 to HighDIO div 8 do begin
    SetByte(I);
  end;

  for I := 0 to High(ClockButton) do begin
    ClockButton[I].AllowAllUp := True;
    ClockButton[I].GroupIndex := I + 1;
    SetClock(I);
  end;

  Caption := 'C=' + IntToBin(InPortB(Base + 2), 8);

  CommandEdit.Visible := True;
  //Caption := 'Loaded, base address ' + IntToHex(Base, 1);
end;

procedure TMainForm.CommandEditKeyPress(Sender: TObject; var Key: Char);
var
  B, I: Integer;

begin
  case Key of
    '+': ;
    '-': ;
    'A', 'a': with ClockButton[0] do begin Down := not Down; Click; Key := #0; Exit; end;
    'B', 'b': with ClockButton[1] do begin Down := not Down; Click; Key := #0; Exit; end;
    'C', 'c': with ClockButton[2] do begin Down := not Down; Click; Key := #0; Exit; end;
    else Exit;
  end;
  B := Ord(Key = '+');
  Key := #0;

  I := StrToInt(CommandEdit.Text);

  DIOLabel[I].Tag := B;
  SetByte(I div 8);
  DIOImage[I].Picture.Assign(LevelBitmap[B]);
  CommandEdit.SelectAll;
end;

procedure TMainForm.DIOImageClick(Sender: TObject);
var
  I: Integer;

begin
  I := (Sender as TImage).Tag;

  if DIOLabel[I].Tag <> 0 then begin
    DIOLabel[I].Tag := 0;
    SetByte(I div 8);
    DIOImage[I].Picture.Assign(LevelBitmap[0]);
  end
  else begin
    DIOLabel[I].Tag := 1;
    SetByte(I div 8);
    DIOImage[I].Picture.Assign(LevelBitmap[1]);
  end;
end;

function CalcAddrMask(I: Integer): Byte;
begin
  Result :=              1 shl (I mod 4) ;
  I      := I div 4;
  Result := Result or ($10 shl (I mod 3));
  I      := I div 3;
  if I <> 0 then Result := Result or $80;

  //Result := not Result;
end;

procedure TMainForm.SetByte(Index: Integer);
var
  Data: Byte;
  I: Integer;
  //Buf: String;

begin
  Data := 0;
  for I := Index * 8 + 7 downto Index * 8 do
    Data := (Data shl 1) or DIOLabel[I].Tag
  ;

  //Buf := 'Write byte: A=' + IntToBin(Data, 8);
  OutPortB(Base + 0, Data);
  repeat until InPortB(Base + 0) = Data;
  Data := CalcAddrMask(Index);
  //Buf := Buf + ', C=' + IntToBin(Data, 8);
  OutPortB(Base + 2, (Data and $80) xor ActivityMask);
  OutPortB(Base + 2, (Data and $F0) xor ActivityMask);
  OutPortB(Base + 2,  Data          xor ActivityMask);
  OutPortB(Base + 2, (Data and $F0) xor ActivityMask);
  OutPortB(Base + 2, (Data and $80) xor ActivityMask);
  OutPortB(Base + 2, ActivityMask);
  OutPortB(Base + 0, $FF);
  //Caption := Buf;
end;

procedure CtrMode(Base: Word; Ctr, Mode: Byte);
begin
  OutPortB(Base+3, (Ctr shl 6) or $30 or (Mode shl 1));
end;

procedure TMainForm.SetClock(Index: Integer);
begin
  CtrMode(Base + $10 + $4 * Index, 2, Ord(not ClockButton[Index].Down));
end;

procedure TMainForm.ClockButtonClick(Sender: TObject);
begin
  SetClock((Sender as TSpeedButton).Tag);
end;

end.
